@@ -24,6 +24,7 @@
 
 #include "gtkintl.h"
 #include "gtkcellrenderertext.h"
+#include "gtkframe.h"
 #include "gtktreeselection.h"
 #include "gtktreeview.h"
 #include "gtkscrolledwindow.h"
@@ -254,6 +255,7 @@ gtk_entry_completion_init (GtkEntryCompletion *completion)
   GtkCellRenderer *cell;
   GtkTreeSelection *sel;
   GtkEntryCompletionPrivate *priv;
+  GtkWidget *popup_frame;
 
   /* yes, also priv, need to keep the code readable */
   priv = completion->priv = GTK_ENTRY_COMPLETION_GET_PRIVATE (completion);
@@ -286,7 +288,7 @@ gtk_entry_completion_init (GtkEntryCompletion *completion)
                                   GTK_POLICY_NEVER,
                                   GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (priv->scrolled_window),
-                                       GTK_SHADOW_ETCHED_IN);
+                                       GTK_SHADOW_NONE);
 
   /* a nasty hack to get the completions treeview to size nicely */
   gtk_widget_set_size_request (GTK_SCROLLED_WINDOW (priv->scrolled_window)->vscrollbar, -1, 0);
@@ -323,8 +325,14 @@ gtk_entry_completion_init (GtkEntryCompletion *completion)
                     G_CALLBACK (gtk_entry_completion_popup_button_press),
                     completion);
 
+  popup_frame = gtk_frame_new (NULL);
+  gtk_frame_set_shadow_type (GTK_FRAME (popup_frame),
+			     GTK_SHADOW_ETCHED_IN);
+  gtk_widget_show (popup_frame);
+  gtk_container_add (GTK_CONTAINER (priv->popup_window), popup_frame);
+  
   priv->vbox = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (priv->popup_window), priv->vbox);
+  gtk_container_add (GTK_CONTAINER (popup_frame), priv->vbox);
 
   gtk_container_add (GTK_CONTAINER (priv->scrolled_window), priv->tree_view);
   gtk_box_pack_start (GTK_BOX (priv->vbox), priv->scrolled_window,
@@ -1086,16 +1094,27 @@ get_borders (GtkEntry *entry,
 }
 
 /* some nasty size requisition */
-gint
+gboolean
 _gtk_entry_completion_resize_popup (GtkEntryCompletion *completion)
 {
-  gint items, height, x_border, y_border;
+  gint x, y;
+  gint matches, items, height, x_border, y_border;
+  GdkScreen *screen;
+  gint monitor_num;
+  GdkRectangle monitor;
+  GtkRequisition popup_req;
+  GtkTreePath *path;
+  gboolean above;
 
+  gdk_window_get_origin (completion->priv->entry->window, &x, &y);
   get_borders (GTK_ENTRY (completion->priv->entry), &x_border, &y_border);
 
-  items = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (completion->priv->filter_model), NULL);
+  x += x_border;
+  y += 2 * y_border;
 
-  items = MIN (items, 15);
+  matches = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (completion->priv->filter_model), NULL);
+
+  items = MIN (matches, 15);
 
   gtk_tree_view_column_cell_get_size (completion->priv->column, NULL,
                                       NULL, NULL, NULL, &height);
@@ -1129,42 +1148,6 @@ _gtk_entry_completion_resize_popup (GtkEntryCompletion *completion)
   else
     gtk_widget_hide (completion->priv->action_view);
 
-  return height;
-}
-
-void
-_gtk_entry_completion_popup (GtkEntryCompletion *completion)
-{
-  gint x, y, x_border, y_border;
-  gint height;
-  GdkScreen *screen;
-  gint monitor_num;
-  GdkRectangle monitor;
-  GtkRequisition popup_req;
-  GtkTreeViewColumn *column;
-  GList *renderers;
-
-  if (GTK_WIDGET_MAPPED (completion->priv->popup_window))
-    return;
-
-  column = gtk_tree_view_get_column (GTK_TREE_VIEW (completion->priv->action_view), 0);
-  renderers = gtk_tree_view_column_get_cell_renderers (column);
-  gtk_widget_ensure_style (completion->priv->tree_view);
-  g_object_set (GTK_CELL_RENDERER (renderers->data), "cell_background_gdk",
-                &completion->priv->tree_view->style->bg[GTK_STATE_NORMAL],
-                NULL);
-  g_list_free (renderers);
-
-  gtk_widget_show_all (completion->priv->vbox);
-
-  gdk_window_get_origin (completion->priv->entry->window, &x, &y);
-  get_borders (GTK_ENTRY (completion->priv->entry), &x_border, &y_border);
-
-  x += x_border;
-  y += 2 * y_border;
-
-  height = _gtk_entry_completion_resize_popup (completion);
-
   gtk_widget_size_request (completion->priv->popup_window, &popup_req);
 
   screen = gtk_widget_get_screen (GTK_WIDGET (completion->priv->entry));
@@ -1178,12 +1161,52 @@ _gtk_entry_completion_popup (GtkEntryCompletion *completion)
     x = monitor.x + monitor.width - popup_req.width;
   
   if (y + height + popup_req.height <= monitor.y + monitor.height)
-    y += height;
+    {
+      y += height;
+      above = FALSE;
+    }
   else
-    y -= popup_req.height;
+    {
+      y -= popup_req.height;
+      above = TRUE;
+    }
+  
+  if (matches > 0) 
+    {
+      path = gtk_tree_path_new_from_indices (above ? matches - 1 : 0, -1);
+      gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (completion->priv->tree_view), path, 
+				    NULL, FALSE, 0.0, 0.0);
+      gtk_tree_path_free (path);
+    }
 
   gtk_window_move (GTK_WINDOW (completion->priv->popup_window), x, y);
 
+  return above;
+}
+
+void
+_gtk_entry_completion_popup (GtkEntryCompletion *completion)
+{
+  GtkTreeViewColumn *column;
+  GList *renderers;
+
+  if (GTK_WIDGET_MAPPED (completion->priv->popup_window))
+    return;
+
+  completion->priv->may_wrap = TRUE;
+
+  column = gtk_tree_view_get_column (GTK_TREE_VIEW (completion->priv->action_view), 0);
+  renderers = gtk_tree_view_column_get_cell_renderers (column);
+  gtk_widget_ensure_style (completion->priv->tree_view);
+  g_object_set (GTK_CELL_RENDERER (renderers->data), "cell_background_gdk",
+                &completion->priv->tree_view->style->bg[GTK_STATE_NORMAL],
+                NULL);
+  g_list_free (renderers);
+
+  gtk_widget_show_all (completion->priv->vbox);
+
+  _gtk_entry_completion_resize_popup (completion);
+
   gtk_widget_show (completion->priv->popup_window);
 
   gtk_grab_add (completion->priv->popup_window);
