@@ -18,11 +18,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include <string.h>
 
 #include "gtkfilesystemmodel.h"
 #include "gtkfilesystem.h"
 #include "gtkintl.h"
+#include "gtkmarshalers.h"
+#include "gtktreednd.h"
 #include "gtktreemodel.h"
 
 typedef struct _GtkFileSystemModelClass GtkFileSystemModelClass;
@@ -35,6 +38,10 @@ typedef struct _FileModelNode           FileModelNode;
 struct _GtkFileSystemModelClass
 {
   GObjectClass parent_class;
+
+  /* Signals */
+
+  void (*finished_loading) (GtkFileSystemModel *model);
 };
 
 struct _GtkFileSystemModel
@@ -52,6 +59,7 @@ struct _GtkFileSystemModel
 
   GSList *idle_clears;
   GSource *idle_clear_source;
+  GSource *idle_finished_loading_source;
 
   gushort max_depth;
   
@@ -91,6 +99,8 @@ static void gtk_file_system_model_iface_init   (GtkTreeModelIface       *iface);
 static void gtk_file_system_model_init         (GtkFileSystemModel      *model);
 static void gtk_file_system_model_finalize     (GObject                 *object);
 
+static void drag_source_iface_init (GtkTreeDragSourceIface *iface);
+
 static GtkTreeModelFlags gtk_file_system_model_get_flags       (GtkTreeModel *tree_model);
 static gint              gtk_file_system_model_get_n_columns   (GtkTreeModel *tree_model);
 static GType             gtk_file_system_model_get_column_type (GtkTreeModel *tree_model,
@@ -125,6 +135,12 @@ static void              gtk_file_system_model_ref_node        (GtkTreeModel *tr
 static void              gtk_file_system_model_unref_node      (GtkTreeModel *tree_model,
 								GtkTreeIter  *iter);
 
+static gboolean drag_source_row_draggable (GtkTreeDragSource   *drag_source,
+					   GtkTreePath         *path);
+static gboolean drag_source_drag_data_get (GtkTreeDragSource   *drag_source,
+					   GtkTreePath         *path,
+					   GtkSelectionData    *selection_data);
+
 static FileModelNode *file_model_node_new        (GtkFileSystemModel *model,
 						  const GtkFilePath  *path);
 static void           file_model_node_free       (FileModelNode      *node);
@@ -176,6 +192,16 @@ static void root_files_removed_callback (GtkFileFolder      *folder,
 
 static GObjectClass *parent_class = NULL;
 
+/* Signal IDs */
+enum {
+  FINISHED_LOADING,
+  LAST_SIGNAL
+};
+
+static guint file_system_model_signals[LAST_SIGNAL] = { 0 };
+
+
+
 GType
 _gtk_file_system_model_get_type (void)
 {
@@ -199,8 +225,15 @@ _gtk_file_system_model_get_type (void)
       static const GInterfaceInfo file_system_info =
       {
 	(GInterfaceInitFunc) gtk_file_system_model_iface_init, /* interface_init */
-	NULL,			                              /* interface_finalize */
-	NULL			                              /* interface_data */
+	NULL,			                               /* interface_finalize */
+	NULL			                               /* interface_data */
+      };
+
+      static const GInterfaceInfo drag_source_info =
+      {
+	(GInterfaceInitFunc) drag_source_iface_init,           /* interface_init */
+	NULL,			                               /* interface_finalize */
+	NULL			                               /* interface_data */
       };
 
       file_system_model_type = g_type_register_static (G_TYPE_OBJECT,
@@ -209,6 +242,9 @@ _gtk_file_system_model_get_type (void)
       g_type_add_interface_static (file_system_model_type,
 				   GTK_TYPE_TREE_MODEL,
 				   &file_system_info);
+      g_type_add_interface_static (file_system_model_type,
+				   GTK_TYPE_TREE_DRAG_SOURCE,
+				   &drag_source_info);
     }
 
   return file_system_model_type;
@@ -222,6 +258,15 @@ gtk_file_system_model_class_init (GtkFileSystemModelClass *class)
   parent_class = g_type_class_peek_parent (class);
 
   gobject_class->finalize = gtk_file_system_model_finalize;
+
+  file_system_model_signals[FINISHED_LOADING] =
+    g_signal_new ("finished-loading",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (GtkFileSystemModelClass, finished_loading),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
 }
 
 static void
@@ -266,6 +311,9 @@ gtk_file_system_model_finalize (GObject *object)
   if (model->file_system)
     g_object_unref (model->file_system);
 
+  if (model->idle_finished_loading_source)
+    g_source_destroy (model->idle_finished_loading_source);
+
   children = model->roots;
   while (children)
     {
@@ -277,6 +325,14 @@ gtk_file_system_model_finalize (GObject *object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+drag_source_iface_init (GtkTreeDragSourceIface *iface)
+{
+  iface->row_draggable = drag_source_row_draggable;
+  iface->drag_data_get = drag_source_drag_data_get;
+  iface->drag_data_delete = NULL;
+}
+
 /*
  * ******************** GtkTreeModel methods ********************
  */
@@ -287,7 +343,7 @@ gtk_file_system_model_get_flags (GtkTreeModel *tree_model)
   GtkFileSystemModel *model = GTK_FILE_SYSTEM_MODEL (tree_model);
   GtkTreeModelFlags flags = GTK_TREE_MODEL_ITERS_PERSIST;
 
-  if (model->max_depth == 1)
+  if (model->max_depth == 0)
     flags |= GTK_TREE_MODEL_LIST_ONLY;
 
   return flags;
@@ -565,6 +621,93 @@ gtk_file_system_model_unref_node (GtkTreeModel *tree_model,
 			 iter->user_data);
 }
 
+static gboolean
+drag_source_row_draggable (GtkTreeDragSource *drag_source,
+			   GtkTreePath       *path)
+{
+  GtkFileSystemModel *model;
+  GtkTreeIter iter;
+  FileModelNode *node;
+
+  model = GTK_FILE_SYSTEM_MODEL (drag_source);
+
+  if (!gtk_file_system_model_get_iter (GTK_TREE_MODEL (model), &iter, path))
+    return FALSE;
+
+  if (!model->has_editable)
+    return TRUE;
+
+  node = iter.user_data;
+  return (node != model->roots);
+}
+
+static gboolean
+drag_source_drag_data_get (GtkTreeDragSource *drag_source,
+			   GtkTreePath       *path,
+			   GtkSelectionData  *selection_data)
+{
+  GtkFileSystemModel *model;
+  GtkTreeIter iter;
+  const GtkFilePath *file_path;
+  char *uri;
+  char *uris;
+
+  model = GTK_FILE_SYSTEM_MODEL (drag_source);
+
+  if (!gtk_file_system_model_get_iter (GTK_TREE_MODEL (model), &iter, path))
+    return FALSE;
+
+  file_path = _gtk_file_system_model_get_path (model, &iter);
+  g_assert (file_path != NULL);
+
+  uri = gtk_file_system_path_to_uri (model->file_system, file_path);
+  uris = g_strconcat (uri, "\r\n", NULL);
+
+  gtk_selection_data_set (selection_data,
+			  gdk_atom_intern ("text/uri-list", FALSE),
+			  8,
+			  uris,
+			  strlen (uris) + 1);
+
+  g_free (uri);
+  g_free (uris);
+
+  return TRUE;
+}
+
+/* Callback used when the root folder finished loading */
+static void
+root_folder_finished_loading_cb (GtkFileFolder      *folder,
+				 GtkFileSystemModel *model)
+{
+  g_signal_emit (model, file_system_model_signals[FINISHED_LOADING], 0);
+}
+
+/* Emits the "finished-loading" signal as an idle handler; see the comment in
+ * _gtk_file_system_model_new()
+ */
+static gboolean
+idle_finished_loading_cb (GtkFileSystemModel *model)
+{
+  g_signal_emit (model, file_system_model_signals[FINISHED_LOADING], 0);
+
+  g_source_destroy (model->idle_finished_loading_source);
+  model->idle_finished_loading_source = NULL;
+
+  return FALSE;
+}
+
+/* Queues an idle handler to emit the "finished-loading" signal */
+static void
+queue_finished_loading (GtkFileSystemModel *model)
+{
+  model->idle_finished_loading_source = g_idle_source_new ();
+  g_source_set_closure (model->idle_finished_loading_source,
+			g_cclosure_new_object (G_CALLBACK (idle_finished_loading_cb),
+					       G_OBJECT (model)));
+  g_source_attach (model->idle_finished_loading_source, NULL);
+}
+
 /**
  * _gtk_file_system_model_new:
  * @file_system: an object implementing #GtkFileSystem
@@ -617,13 +760,19 @@ _gtk_file_system_model_new (GtkFileSystem     *file_system,
 						       model->types,
 						       NULL);   /* NULL-GError */
 
-      if (model->root_folder &&
-	  gtk_file_folder_list_children (model->root_folder,
-					 &child_paths,
-					 NULL)) /* NULL-GError */
+      if (model->root_folder)
 	{
-	  roots = child_paths;
-	  
+	  if (gtk_file_folder_list_children (model->root_folder,
+					     &child_paths,
+					     NULL)) /* NULL-GError */
+	    roots = child_paths;
+
+	  if (gtk_file_folder_is_finished_loading (model->root_folder))
+	    queue_finished_loading (model); /* done in an idle because we are being created */
+	  else
+	    g_signal_connect_object (model->root_folder, "finished-loading",
+				     G_CALLBACK (root_folder_finished_loading_cb), model, 0);
+
 	  g_signal_connect_object (model->root_folder, "deleted",
 				   G_CALLBACK (root_deleted_callback), model, 0);
 	  g_signal_connect_object (model->root_folder, "files-added",
@@ -700,9 +849,8 @@ model_refilter_recurse (GtkFileSystemModel *model,
 	  GtkTreeIter iter;
 
 	  iter.user_data = nodes;
-	  gtk_tree_model_row_inserted (tree_model, path, &iter);
-
 	  nodes->is_visible = TRUE;
+	  gtk_tree_model_row_inserted (tree_model, path, &iter);
 	}
       else
 	model_refilter_recurse (model, nodes, path);
@@ -1147,24 +1295,32 @@ static gboolean
 file_model_node_is_visible (GtkFileSystemModel *model,
 			    FileModelNode      *node)
 {
-  if (model->show_hidden && model->show_folders && model->show_files)
-    return TRUE;
-  else
+  if (model->show_folders != model->show_files ||
+      !model->show_hidden ||
+      model->filter_func)
     {
       const GtkFileInfo *info = file_model_node_get_info (model, node);
-      gboolean is_folder = gtk_file_info_get_is_folder (info);
 
-      if (!model->show_folders && is_folder)
-	return FALSE;
-      if (!model->show_files && !is_folder)
+      if (!info)
+	{
+	  /* File probably disappeared underneath us or resides in a
+	     directory where we have only partial access rights.  */
+	  return FALSE;
+	}
+
+      if (model->show_folders != model->show_files &&
+	  model->show_folders != gtk_file_info_get_is_folder (info))
 	return FALSE;
+
       if (!model->show_hidden && gtk_file_info_get_is_hidden (info))
 	return FALSE;
-      if (model->filter_func && !model->filter_func (model, node->path, info, model->filter_data))
-	return FALSE;
 
-      return TRUE;
+      if (model->filter_func &&
+	  !model->filter_func (model, node->path, info, model->filter_data))
+	return FALSE;
     }
+
+  return TRUE;
 }
 
 static void
